import deleteFollowing from "@/services/following/delete.js";
import define from "../../define.js";
import { ApiError } from "../../error.js";
import { getUser } from "../../common/getters.js";
import { Followings, Users } from "@/models/index.js";
import { HOUR } from "@/const.js";

export const meta = {
	tags: ["following", "users"],

	limit: {
		duration: HOUR,
		max: 100,
	},

	requireCredential: true,

	kind: "write:following",

	errors: {
		noSuchUser: {
			message: "No such user.",
			code: "NO_SUCH_USER",
			id: "5b12c78d-2b28-4dca-99d2-f56139b42ff8",
		},

		followeeIsYourself: {
			message: "Followee is yourself.",
			code: "FOLLOWEE_IS_YOURSELF",
			id: "d9e400b9-36b0-4808-b1d8-79e707f1296c",
		},

		notFollowing: {
			message: "You are not following that user.",
			code: "NOT_FOLLOWING",
			id: "5dbf82f5-c92b-40b1-87d1-6c8c0741fd09",
		},
	},

	res: {
		type: "object",
		optional: false,
		nullable: false,
		ref: "UserLite",
	},
} as const;

export const paramDef = {
	type: "object",
	properties: {
		userId: { type: "string", format: "misskey:id" },
	},
	required: ["userId"],
} as const;

export default define(meta, paramDef, async (ps, user) => {
	const follower = user;

	// Check if the followee is yourself
	if (user.id === ps.userId) {
		throw new ApiError(meta.errors.followeeIsYourself);
	}

	// Get followee
	const followee = await getUser(ps.userId).catch((e) => {
		if (e.id === "15348ddd-432d-49c2-8a5a-8069753becff")
			throw new ApiError(meta.errors.noSuchUser);
		throw e;
	});

	// Check not following
	const exist = await Followings.exist({
		where: {
			followerId: follower.id,
			followeeId: followee.id,
		},
	});

	if (!exist) {
		throw new ApiError(meta.errors.notFollowing);
	}

	await deleteFollowing(follower, followee);

	return await Users.pack(followee.id, user);
});
